@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,23 +14,25 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -275,6 +277,73 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -325,50 +394,22 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, ManagedIdentitiesServiceTransport):
             # transport is a ManagedIdentitiesServiceTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -380,6 +421,15 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -407,6 +457,36 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> operation.Operation:
         r"""Creates a Microsoft AD domain.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_create_microsoft_ad_domain():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                domain = managedidentities_v1.Domain()
+                domain.name = "name_value"
+                domain.reserved_ip_range = "reserved_ip_range_value"
+                domain.locations = ['locations_value_1', 'locations_value_2']
+
+                request = managedidentities_v1.CreateMicrosoftAdDomainRequest(
+                    parent="parent_value",
+                    domain_name="domain_name_value",
+                    domain=domain,
+                )
+
+                # Make the request
+                operation = client.create_microsoft_ad_domain(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.CreateMicrosoftAdDomainRequest, dict]):
                 The request object. Request message for
@@ -460,7 +540,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, domain_name, domain])
         if request is not None and has_flattened_params:
@@ -525,6 +605,25 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> managed_identities_service.ResetAdminPasswordResponse:
         r"""Resets a domain's administrator password.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_reset_admin_password():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                request = managedidentities_v1.ResetAdminPasswordRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.reset_admin_password(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.ResetAdminPasswordRequest, dict]):
                 The request object. Request message for
@@ -549,7 +648,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -598,6 +697,26 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> pagers.ListDomainsPager:
         r"""Lists domains in a project.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_list_domains():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                request = managedidentities_v1.ListDomainsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_domains(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.ListDomainsRequest, dict]):
                 The request object. Request message for
@@ -625,7 +744,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -678,6 +797,25 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> resource.Domain:
         r"""Gets information about a domain.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_get_domain():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                request = managedidentities_v1.GetDomainRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_domain(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.GetDomainRequest, dict]):
                 The request object. Request message for
@@ -702,7 +840,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -750,6 +888,34 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> operation.Operation:
         r"""Updates the metadata and configuration of a domain.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_update_domain():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                domain = managedidentities_v1.Domain()
+                domain.name = "name_value"
+                domain.reserved_ip_range = "reserved_ip_range_value"
+                domain.locations = ['locations_value_1', 'locations_value_2']
+
+                request = managedidentities_v1.UpdateDomainRequest(
+                    domain=domain,
+                )
+
+                # Make the request
+                operation = client.update_domain(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.UpdateDomainRequest, dict]):
                 The request object. Request message for
@@ -790,7 +956,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([domain, update_mask])
         if request is not None and has_flattened_params:
@@ -849,6 +1015,29 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> operation.Operation:
         r"""Deletes a domain.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_delete_domain():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                request = managedidentities_v1.DeleteDomainRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                operation = client.delete_domain(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.DeleteDomainRequest, dict]):
                 The request object. Request message for
@@ -886,7 +1075,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -942,6 +1131,37 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> operation.Operation:
         r"""Adds an AD trust to a domain.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_attach_trust():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                trust = managedidentities_v1.Trust()
+                trust.target_domain_name = "target_domain_name_value"
+                trust.trust_type = "EXTERNAL"
+                trust.trust_direction = "BIDIRECTIONAL"
+                trust.target_dns_ip_addresses = ['target_dns_ip_addresses_value_1', 'target_dns_ip_addresses_value_2']
+                trust.trust_handshake_secret = "trust_handshake_secret_value"
+
+                request = managedidentities_v1.AttachTrustRequest(
+                    name="name_value",
+                    trust=trust,
+                )
+
+                # Make the request
+                operation = client.attach_trust(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.AttachTrustRequest, dict]):
                 The request object. Request message for
@@ -975,7 +1195,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name, trust])
         if request is not None and has_flattened_params:
@@ -1034,6 +1254,31 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> operation.Operation:
         r"""Updates the DNS conditional forwarder.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_reconfigure_trust():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                request = managedidentities_v1.ReconfigureTrustRequest(
+                    name="name_value",
+                    target_domain_name="target_domain_name_value",
+                    target_dns_ip_addresses=['target_dns_ip_addresses_value_1', 'target_dns_ip_addresses_value_2'],
+                )
+
+                # Make the request
+                operation = client.reconfigure_trust(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.ReconfigureTrustRequest, dict]):
                 The request object. Request message for
@@ -1078,7 +1323,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name, target_domain_name, target_dns_ip_addresses])
         if request is not None and has_flattened_params:
@@ -1138,6 +1383,37 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
     ) -> operation.Operation:
         r"""Removes an AD trust.
 
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_detach_trust():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                trust = managedidentities_v1.Trust()
+                trust.target_domain_name = "target_domain_name_value"
+                trust.trust_type = "EXTERNAL"
+                trust.trust_direction = "BIDIRECTIONAL"
+                trust.target_dns_ip_addresses = ['target_dns_ip_addresses_value_1', 'target_dns_ip_addresses_value_2']
+                trust.trust_handshake_secret = "trust_handshake_secret_value"
+
+                request = managedidentities_v1.DetachTrustRequest(
+                    name="name_value",
+                    trust=trust,
+                )
+
+                # Make the request
+                operation = client.detach_trust(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.DetachTrustRequest, dict]):
                 The request object. Request message for
@@ -1173,7 +1449,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name, trust])
         if request is not None and has_flattened_params:
@@ -1233,6 +1509,38 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
         reachable, and that the target domain is able to accept
         incoming trust requests.
 
+
+        .. code-block:: python
+
+            from google.cloud import managedidentities_v1
+
+            def sample_validate_trust():
+                # Create a client
+                client = managedidentities_v1.ManagedIdentitiesServiceClient()
+
+                # Initialize request argument(s)
+                trust = managedidentities_v1.Trust()
+                trust.target_domain_name = "target_domain_name_value"
+                trust.trust_type = "EXTERNAL"
+                trust.trust_direction = "BIDIRECTIONAL"
+                trust.target_dns_ip_addresses = ['target_dns_ip_addresses_value_1', 'target_dns_ip_addresses_value_2']
+                trust.trust_handshake_secret = "trust_handshake_secret_value"
+
+                request = managedidentities_v1.ValidateTrustRequest(
+                    name="name_value",
+                    trust=trust,
+                )
+
+                # Make the request
+                operation = client.validate_trust(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.managedidentities_v1.types.ValidateTrustRequest, dict]):
                 The request object. Request message for
@@ -1268,7 +1576,7 @@ class ManagedIdentitiesServiceClient(metaclass=ManagedIdentitiesServiceClientMet
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name, trust])
         if request is not None and has_flattened_params:
